import asyncio
from collections.abc import Iterable, Sequence
from enum import Enum
from functools import cache
from pathlib import Path
import sys

from anyio import Path as APath
from hishel import AsyncCacheClient, AsyncFileStorage
from httpx import Headers
import humanize
import inflect
from packaging.specifiers import SpecifierSet
from rich.console import Console, ConsoleRenderable
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

from uv_secure import __version__
from uv_secure.configuration import (
    config_cli_arg_factory,
    config_file_factory,
    Configuration,
    override_config,
)
from uv_secure.directory_scanner import get_dependency_file_to_config_map
from uv_secure.package_info import (
    download_packages,
    PackageInfo,
    parse_requirements_txt_file,
    parse_uv_lock_file,
)


if sys.version_info < (3, 11):
    from exceptiongroup import ExceptionGroup


USER_AGENT = f"uv-secure/{__version__} (contact: owenrlamont@gmail.com)"


def _render_vulnerability_table(
    config: Configuration, vulnerable_packages: Iterable[PackageInfo]
) -> Table:
    table = Table(
        title="Vulnerable Dependencies",
        show_header=True,
        row_styles=["none", "dim"],
        header_style="bold magenta",
        expand=True,
    )
    table.add_column("Package", min_width=8, max_width=40)
    table.add_column("Version", min_width=10, max_width=20)
    table.add_column("Vulnerability ID", style="bold cyan", min_width=20, max_width=24)
    table.add_column("Fix Versions", min_width=10, max_width=20)
    if config.vulnerability_criteria.aliases:
        table.add_column("Aliases", min_width=20, max_width=24)
    if config.vulnerability_criteria.desc:
        table.add_column("Details", min_width=8)
    for package in vulnerable_packages:
        for vuln in package.vulnerabilities:
            vuln_id_hyperlink = (
                Text.assemble((vuln.id, f"link {vuln.link}"))
                if vuln.link
                else Text(vuln.id)
            )
            renderables = [
                Text.assemble(
                    (
                        package.info.name,
                        f"link https://pypi.org/project/{package.info.name}",
                    )
                ),
                Text.assemble(
                    (
                        package.info.version,
                        f"link https://pypi.org/project/{package.info.name}/"
                        f"{package.info.version}/",
                    )
                ),
                vuln_id_hyperlink,
                Text(", ").join(
                    [
                        Text.assemble(
                            (
                                fix_ver,
                                f"link https://pypi.org/project/{package.info.name}/"
                                f"{fix_ver}/",
                            )
                        )
                        for fix_ver in vuln.fixed_in
                    ]
                )
                if vuln.fixed_in
                else Text(""),
            ]
            if config.vulnerability_criteria.aliases:
                alias_links = []
                for alias in vuln.aliases or []:
                    hyperlink = None
                    if alias.startswith("CVE-"):
                        hyperlink = (
                            f"https://cve.mitre.org/cgi-bin/cvename.cgi?name={alias}"
                        )
                    elif alias.startswith("GHSA-"):
                        hyperlink = f"https://github.com/advisories/{alias}"
                    elif alias.startswith("PYSEC-"):
                        hyperlink = (
                            "https://github.com/pypa/advisory-database/blob/main/"
                            f"vulns/{package.info.name}/{alias}.yaml"
                        )
                    elif alias.startswith("OSV-"):
                        hyperlink = f"https://osv.dev/vulnerability/{alias}"
                    if hyperlink:
                        alias_links.append(Text.assemble((alias, f"link {hyperlink}")))
                    else:
                        alias_links.append(Text(alias))
                renderables.append(
                    Text(", ").join(alias_links) if alias_links else Text("")
                )
            if config.vulnerability_criteria.desc:
                renderables.append(vuln.details)
            table.add_row(*renderables)
    return table


def _render_issue_table(
    config: Configuration, maintenance_issue_packages: Iterable[PackageInfo]
) -> Table:
    table = Table(
        title="Maintenance Issues",
        show_header=True,
        row_styles=["none", "dim"],
        header_style="bold magenta",
        expand=True,
    )
    table.add_column("Package", min_width=8, max_width=40)
    table.add_column("Version", min_width=10, max_width=20)
    table.add_column("Yanked", style="bold cyan", min_width=10, max_width=10)
    table.add_column("Yanked Reason", min_width=20, max_width=24)
    table.add_column("Age", min_width=20, max_width=24)
    for package in maintenance_issue_packages:
        renderables = [
            Text.assemble(
                (
                    package.info.name,
                    f"link https://pypi.org/project/{package.info.name}",
                )
            ),
            Text.assemble(
                (
                    package.info.version,
                    f"link https://pypi.org/project/{package.info.name}/"
                    f"{package.info.version}/",
                )
            ),
            str(package.info.yanked),
            package.info.yanked_reason if package.info.yanked_reason else "Unknown",
            humanize.precisedelta(package.age, minimum_unit="days")
            if package.age
            else "Unknown",
        ]
        table.add_row(*renderables)
    return table


@cache
def get_specifier_sets(specifiers: tuple[str, ...]) -> tuple[SpecifierSet, ...]:
    """Converts a tuple of version specifiers to a tuple of SpecifierSets

    Args:
        specifiers: tuple of version specifiers

    Returns:
        tuple of SpecifierSets
    """
    return tuple(SpecifierSet(spec) for spec in specifiers)


async def check_dependencies(
    dependency_file_path: APath,
    config: Configuration,
    http_client: AsyncCacheClient,
    disable_cache: bool,
) -> tuple[int, Iterable[ConsoleRenderable]]:
    """Checks dependencies for vulnerabilities and summarizes the results

    Args:
        dependency_file_path: uv.lock file or requirements.txt file path
        config: uv-secure configuration object
        http_client: HTTP client for making requests
        disable_cache: flag whether to disable cache for HTTP requests

    Returns:
        tuple with status code and output for console to render
    """
    console_outputs = []

    if not await dependency_file_path.exists():
        console_outputs.append(
            f"[bold red]Error:[/] File {dependency_file_path} does not exist."
        )
        return 3, console_outputs

    if dependency_file_path.name == "uv.lock":
        dependencies = await parse_uv_lock_file(dependency_file_path)
    else:  # Assume dependency_file_path.name == "requirements.txt"
        dependencies = await parse_requirements_txt_file(dependency_file_path)

    if len(dependencies) == 0:
        return 0, console_outputs

    console_outputs.append(
        f"[bold cyan]Checking {dependency_file_path} dependencies for vulnerabilities"
        "...[/]\n"
    )

    packages = await download_packages(dependencies, http_client, disable_cache)

    total_dependencies = len(packages)
    vulnerable_count = 0
    vulnerable_packages = []
    maintenance_issue_packages = []

    ignore_packages = {}
    if config.ignore_packages is not None:
        ignore_packages = {
            name: get_specifier_sets(tuple(specifiers))
            for name, specifiers in config.ignore_packages.items()
        }

    for idx, package in enumerate(packages):
        if isinstance(package, BaseException):
            console_outputs.append(
                f"[bold red]Error:[/] {dependencies[idx]} raised exception: {package}"
            )
            return 3, console_outputs

        if package.info.name in ignore_packages:
            specifiers: tuple[SpecifierSet, ...] = ignore_packages[package.info.name]
            if len(specifiers) == 0 or any(
                specifier.contains(package.info.version) for specifier in specifiers
            ):
                console_outputs.append(
                    f"[bold yellow]Skipping {package.info.name} "
                    f"({package.info.version}) as it is ignored[/]"
                )
                continue

        if (
            package.direct_dependency
            or not config.vulnerability_criteria.check_direct_dependencies_only
        ):
            # Filter out ignored vulnerabilities
            package.vulnerabilities = [
                vuln
                for vuln in package.vulnerabilities
                if (
                    config.vulnerability_criteria.ignore_vulnerabilities is None
                    or vuln.id
                    not in config.vulnerability_criteria.ignore_vulnerabilities
                )
                and vuln.withdrawn is None
            ]
            if len(package.vulnerabilities) > 0:
                vulnerable_count += len(package.vulnerabilities)
                vulnerable_packages.append(package)

        if (
            package.direct_dependency
            or not config.maintainability_criteria.check_direct_dependencies_only
        ):
            found_rejected_yanked_package = (
                config.maintainability_criteria.forbid_yanked and package.info.yanked
            )
            found_over_age_package = (
                config.maintainability_criteria.max_package_age is not None
                and package.age is not None
                and package.age > config.maintainability_criteria.max_package_age
            )
            if found_rejected_yanked_package or found_over_age_package:
                maintenance_issue_packages.append(package)

    inf = inflect.engine()
    total_plural = inf.plural("dependency", total_dependencies)
    vulnerable_plural = inf.plural("vulnerability", vulnerable_count)

    status = 0
    if vulnerable_count > 0:
        console_outputs.append(
            Panel.fit(
                f"[bold red]Vulnerabilities detected![/]\n"
                f"Checked: [bold]{total_dependencies}[/] {total_plural}\n"
                f"Vulnerable: [bold]{vulnerable_count}[/] {vulnerable_plural}"
            )
        )
        table = _render_vulnerability_table(config, vulnerable_packages)
        console_outputs.append(table)
        status = 2

    issue_count = len(maintenance_issue_packages)
    issue_plural = inf.plural("issue", issue_count)
    if len(maintenance_issue_packages) > 0:
        console_outputs.append(
            Panel.fit(
                f"[bold yellow]Maintenance Issues detected![/]\n"
                f"Checked: [bold]{total_dependencies}[/] {total_plural}\n"
                f"Issues: [bold]{issue_count}[/] {issue_plural}"
            )
        )
        table = _render_issue_table(config, maintenance_issue_packages)
        console_outputs.append(table)
        status = max(status, 1)

    if status == 0:
        console_outputs.append(
            Panel.fit(
                f"[bold green]No vulnerabilities or maintenance issues detected![/]\n"
                f"Checked: [bold]{total_dependencies}[/] {total_plural}\n"
                f"All dependencies appear safe!"
            )
        )
    return status, console_outputs


class RunStatus(Enum):
    NO_VULNERABILITIES = (0,)
    MAINTENANCE_ISSUES_FOUND = 1
    VULNERABILITIES_FOUND = 2
    RUNTIME_ERROR = 3


async def check_lock_files(
    file_paths: Sequence[Path] | None,
    aliases: bool | None,
    desc: bool | None,
    cache_path: Path,
    cache_ttl_seconds: float,
    disable_cache: bool,
    forbid_yanked: bool | None,
    max_package_age: int | None,
    ignore_vulns: str | None,
    ignore_pkgs: list[str] | None,
    check_direct_dependency_vulnerabilities_only: bool | None,
    check_direct_dependency_maintenance_issues_only: bool | None,
    config_path: Path | None,
) -> RunStatus:
    """Checks uv.lock and requirements.txt files for issues

    Check specified or discovered uv.lock and requirements.txt files for maintenance
    issues or known vulnerabilities

    Args:
        file_paths: paths to files or directory to process
        aliases: flag whether to show vulnerability aliases
        desc: flag whether to show vulnerability descriptions
        cache_path: path to cache directory
        cache_ttl_seconds: time in seconds to cache
        disable_cache: flag whether to disable cache
        forbid_yanked: flag whether to forbid yanked dependencies
        max_package_age: maximum age of dependencies in days
        ignore_vulns: Vulnerabilities IDs to ignore
        ignore_pkgs: list of package names to ignore
        check_direct_dependency_vulnerabilities_only: flag checking direct dependency
            vulnerabilities only
        check_direct_dependency_maintenance_issues_only: flag checking direct dependency
            maintenance issues only
        config_path: path to configuration file


    Returns:
        True if vulnerabilities were found, False otherwise.
    """
    file_apaths: tuple[APath, ...] = (
        (APath(),) if not file_paths else tuple(APath(file) for file in file_paths)
    )

    console = Console()

    try:
        if len(file_apaths) == 1 and await file_apaths[0].is_dir():
            lock_to_config_map = await get_dependency_file_to_config_map(file_apaths[0])
            file_apaths = tuple(lock_to_config_map.keys())
        else:
            if config_path is not None:
                possible_config = await config_file_factory(APath(config_path))
                config = (
                    possible_config if possible_config is not None else Configuration()
                )
                lock_to_config_map = dict.fromkeys(file_apaths, config)
            elif all(
                file_path.name in {"requirements.txt", "uv.lock"}
                for file_path in file_apaths
            ):
                lock_to_config_map = await get_dependency_file_to_config_map(
                    file_apaths
                )
                file_apaths = tuple(lock_to_config_map.keys())
            else:
                console.print(
                    "[bold red]Error:[/] file_paths must either reference a single "
                    "project root directory or a sequence of uv.lock / "
                    "requirements.txt file paths"
                )
                return RunStatus.RUNTIME_ERROR
    except ExceptionGroup as eg:
        for e in eg.exceptions:
            console.print(f"[bold red]Error:[/] {e}")
        return RunStatus.RUNTIME_ERROR

    if any(
        (
            aliases,
            desc,
            ignore_vulns,
            ignore_pkgs,
            forbid_yanked,
            check_direct_dependency_vulnerabilities_only,
            check_direct_dependency_maintenance_issues_only,
            max_package_age is not None,
        )
    ):
        cli_config = config_cli_arg_factory(
            aliases,
            check_direct_dependency_maintenance_issues_only,
            check_direct_dependency_vulnerabilities_only,
            desc,
            forbid_yanked,
            max_package_age,
            ignore_vulns,
            ignore_pkgs,
        )
        lock_to_config_map = {
            lock_file: override_config(config, cli_config)
            for lock_file, config in lock_to_config_map.items()
        }

    # I found antivirus programs (specifically Windows Defender) can almost fully
    # negate the benefits of using a file cache if you don't exclude the virus checker
    # from checking the cache dir given it is frequently read from
    storage = AsyncFileStorage(base_path=cache_path, ttl=cache_ttl_seconds)
    async with AsyncCacheClient(
        timeout=10, headers=Headers({"User-Agent": USER_AGENT}), storage=storage
    ) as http_client:
        status_output_tasks = [
            check_dependencies(
                dependency_file_path,
                lock_to_config_map[APath(dependency_file_path)],
                http_client,
                disable_cache,
            )
            for dependency_file_path in file_apaths
        ]
        status_outputs = await asyncio.gather(*status_output_tasks)
    maintenance_issues_found = False
    vulnerabilities_found = False
    runtime_error = False
    for status, console_output in status_outputs:
        console.print(*console_output)
        if status == 1:
            maintenance_issues_found = True
        elif status == 2:
            vulnerabilities_found = True
        elif status == 3:
            runtime_error = True
    if runtime_error:
        return RunStatus.RUNTIME_ERROR
    if vulnerabilities_found:
        return RunStatus.VULNERABILITIES_FOUND
    if maintenance_issues_found:
        return RunStatus.MAINTENANCE_ISSUES_FOUND
    return RunStatus.NO_VULNERABILITIES



from datetime import timedelta
import sys

from anyio import Path
from pydantic import ValidationError

from uv_secure.configuration.configuration import Configuration, OverrideConfiguration
from uv_secure.configuration.exceptions import UvSecureConfigurationError


if sys.version_info >= (3, 11):
    import tomllib as toml
else:
    import tomli as toml


def _parse_pkg_versions(raw: list[str] | None) -> dict[str, tuple[str, ...]] | None:
    """Parse package and bar delimited version specifiers

    Parse things like "foo:>=1.0,<1.5|==4.5.*" into
    {"foo": [SpecifierSet(">=1.0,<1.5"), SpecifierSet("==4.5.*")]}

    Args:
        raw: list of strings in the format "NAME:SPEC1|SPEC2|…"

    Returns:
        dictionary mapping package names to lists of SpecifierSets

    Raises:
        typer.BadParameter: If the input format is invalid, e.g. missing colon or no
        specifiers
    """
    if not raw:
        return None
    parsed: dict[str, tuple[str, ...]] = {}
    for item in raw:
        if ":" in item:
            name, spec_expr = item.split(":", 1)
            parsed[name] = tuple(spec_expr.split("|"))
        else:
            parsed[item] = ()
    return parsed


def config_cli_arg_factory(
    aliases: bool | None,
    check_direct_dependency_maintenance_issues_only: bool | None,
    check_direct_dependency_vulnerabilities_only: bool | None,
    desc: bool | None,
    forbid_yanked: bool | None,
    max_package_age: int | None,
    ignore_vulns: str | None,
    ignore_pkgs: list[str] | None,
) -> OverrideConfiguration:
    """Factory to create a uv-secure configuration from its command line arguments

    Args:
        aliases: Flag whether to show vulnerability aliases in results
        desc: Flag whether to show vulnerability descriptions in results
        disable_cache: Flag whether to disable cache
        forbid_yanked: flag whether to forbid yanked dependencies
        max_package_age: maximum age of dependencies in days
        ignore_vulns: comma separated string of vulnerability ids to ignore
        ignore_pkgs: list of package names and version specifiers to ignore

    Returns:
        uv-secure override configuration object
    """
    ignore_vulnerabilities = (
        {vuln_id.strip() for vuln_id in ignore_vulns.split(",") if vuln_id.strip()}
        if ignore_vulns is not None
        else None
    )

    return OverrideConfiguration(
        aliases=aliases,
        check_direct_dependency_maintenance_issues_only=check_direct_dependency_maintenance_issues_only,
        check_direct_dependency_vulnerabilities_only=check_direct_dependency_vulnerabilities_only,
        desc=desc,
        forbid_yanked=forbid_yanked,
        max_package_age=timedelta(days=max_package_age) if max_package_age else None,
        ignore_vulnerabilities=ignore_vulnerabilities,
        ignore_packages=_parse_pkg_versions(ignore_pkgs),
    )


async def config_file_factory(config_file: Path) -> Configuration | None:
    """Factory to create a uv-secure configuration from a configuration toml file

    Args:
        config_file: Path to the configuration file (uv-secure.toml, .uv-secure.toml, or
            pyproject.toml)

    Returns:
        uv-secure configuration object or None if no configuration was present
    """
    try:
        config_contents = toml.loads(await config_file.read_text())
        if config_file.name == "pyproject.toml":
            if "tool" in config_contents and "uv-secure" in config_contents["tool"]:
                return Configuration(**config_contents["tool"]["uv-secure"])
            return None
        return Configuration(**config_contents)
    except ValidationError as e:
        raise UvSecureConfigurationError(
            f"Parsing uv-secure configuration at: {config_file} failed. Check the "
            "configuration is up to date as documented at: "
            "https://github.com/owenlamont/uv-secure and check release notes for "
            "breaking changes."
        ) from e



from uv_secure.dependency_checker.dependency_checker import (
    check_dependencies,
    check_lock_files,
    RunStatus,
    USER_AGENT,
)


__all__ = ["USER_AGENT", "RunStatus", "check_dependencies", "check_lock_files"]



from datetime import timedelta

from pydantic import BaseModel, ConfigDict


class MaintainabilityCriteria(BaseModel):
    model_config = ConfigDict(extra="forbid")
    max_package_age: timedelta | None = None
    forbid_yanked: bool = False
    check_direct_dependencies_only: bool = False


class VulnerabilityCriteria(BaseModel):
    model_config = ConfigDict(extra="forbid")
    aliases: bool = False
    desc: bool = False
    ignore_vulnerabilities: set[str] | None = None
    check_direct_dependencies_only: bool = False


class Configuration(BaseModel):
    model_config = ConfigDict(extra="forbid")
    maintainability_criteria: MaintainabilityCriteria = MaintainabilityCriteria()
    vulnerability_criteria: VulnerabilityCriteria = VulnerabilityCriteria()
    ignore_packages: dict[str, tuple[str, ...]] | None = None


class OverrideConfiguration(BaseModel):
    aliases: bool | None = None
    check_direct_dependency_maintenance_issues_only: bool | None = None
    check_direct_dependency_vulnerabilities_only: bool | None = None
    desc: bool | None = None
    ignore_vulnerabilities: set[str] | None = None
    ignore_packages: dict[str, tuple[str, ...]] | None = None
    forbid_yanked: bool | None = None
    max_package_age: timedelta | None = None


def override_config(
    original_config: Configuration, overrides: OverrideConfiguration
) -> Configuration:
    """Override some configuration attributes from an override configuration

    Args:
        original_config: Original unmodified configuration
        overrides: Override attributes to override in original configuration

    Returns:
        Configuration with overridden attributes
    """

    new_configuration = original_config.model_copy()
    if overrides.aliases is not None:
        new_configuration.vulnerability_criteria.aliases = overrides.aliases
    if overrides.check_direct_dependency_maintenance_issues_only is not None:
        new_configuration.maintainability_criteria.check_direct_dependencies_only = (
            overrides.check_direct_dependency_maintenance_issues_only
        )
    if overrides.check_direct_dependency_vulnerabilities_only is not None:
        new_configuration.vulnerability_criteria.check_direct_dependencies_only = (
            overrides.check_direct_dependency_vulnerabilities_only
        )
    if overrides.desc is not None:
        new_configuration.vulnerability_criteria.desc = overrides.desc
    if overrides.ignore_vulnerabilities is not None:
        new_configuration.vulnerability_criteria.ignore_vulnerabilities = (
            overrides.ignore_vulnerabilities
        )
    if overrides.ignore_packages is not None:
        new_configuration.ignore_packages = overrides.ignore_packages
    if overrides.forbid_yanked is not None:
        new_configuration.maintainability_criteria.forbid_yanked = (
            overrides.forbid_yanked
        )
    if overrides.max_package_age is not None:
        new_configuration.maintainability_criteria.max_package_age = (
            overrides.max_package_age
        )

    return new_configuration



import sys

from anyio import Path
from pydantic import BaseModel
import typer


if sys.version_info >= (3, 11):
    import tomllib as toml
else:
    import tomli as toml


class Dependency(BaseModel):
    name: str
    version: str
    direct: bool = False


async def parse_requirements_txt_file(file_path: Path) -> list[Dependency]:
    """Parse a requirements.txt file and extracts package PyPi dependencies"""
    data = await file_path.read_text()
    lines = data.splitlines()
    if (
        len(lines) == 0
        or lines[0].strip()
        != "# This file was autogenerated by uv via the following command:"
    ):
        typer.echo(
            f"Ignoring {file_path} as it doesn't appear to be a uv generated "
            "requirements.txt file"
        )
        return []
    dependencies = []
    dependency: Dependency | None = None
    for line in lines:
        if "==" in line:
            if dependency is not None:
                dependencies.append(dependency)
            name, version = line.split("==")
            dependency = Dependency(name=name.strip(), version=version.strip())
        elif (" -r " in line or " (pyproject.toml)" in line) and dependency is not None:
            dependency.direct = True
    if dependency is not None:
        dependencies.append(dependency)
    return dependencies


async def parse_uv_lock_file(file_path: Path) -> list[Dependency]:
    """Parses a uv.lock TOML file and extracts package PyPi dependencies"""
    data = toml.loads(await file_path.read_text())

    direct_dependencies: set[str] = set()
    dependencies = {}
    package_data = data.get("package", [])
    for package in package_data:
        source = package.get("source", {})
        if source.get("registry") == "https://pypi.org/simple":
            dependencies[package["name"]] = Dependency(
                name=package["name"], version=package["version"]
            )
        elif source.get("editable") == "." or source.get("virtual") == ".":
            for dependency in package.get("dependencies", []):
                direct_dependencies.add(dependency["name"])
            dev_dependencies = package.get("dev-dependencies", {})
            for group_dependencies in dev_dependencies.values():
                for dependency in group_dependencies:
                    direct_dependencies.add(dependency["name"])

    dependency_list = list(dependencies.values())
    for dependency in dependency_list:
        dependency.direct = dependency.name in direct_dependencies
    return dependency_list



from pathlib import Path
import sys

import typer

from uv_secure import __version__
from uv_secure.dependency_checker import check_lock_files, RunStatus


DEFAULT_HTTPX_CACHE_TTL_SECONDS = 24.0 * 60.0 * 60.0


app = typer.Typer()


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"uv-secure {__version__}")
        raise typer.Exit()


_file_path_args = typer.Argument(
    None,
    help=(
        "Paths to the uv.lock or uv generated requirements.txt files or a single "
        "project root level directory (defaults to working directory if not set)"
    ),
)


_aliases_option = typer.Option(
    None,
    "--aliases",
    help="Flag whether to include vulnerability aliases in the vulnerabilities table",
)


_desc_option = typer.Option(
    None,
    "--desc",
    help=(
        "Flag whether to include vulnerability detailed description in the "
        "vulnerabilities table"
    ),
)

_cache_path_option = typer.Option(
    Path.home() / ".cache/uv-secure",
    "--cache-path",
    help="Path to the cache directory for vulnerability http requests",
    show_default="~/.cache/uv-secure",
)

_cache_ttl_seconds_option = typer.Option(
    DEFAULT_HTTPX_CACHE_TTL_SECONDS,
    "--cache-ttl-seconds",
    help="Time to live in seconds for the vulnerability http requests cache",
)

_disable_cache_option = typer.Option(
    False,
    "--disable-cache",
    help="Flag whether to disable caching for vulnerability http requests",
)

_forbid_yanked_option = typer.Option(
    None,
    "--forbid-yanked",
    help="Flag whether disallow yanked package versions from being dependencies",
)

_check_direct_dependency_vulnerabilities_only_option = typer.Option(
    None,
    "--check-direct-dependency-vulnerabilities-only",
    help="Flag whether to only test only direct dependencies for vulnerabilities",
)

_check_direct_dependency_maintenance_issues_only_option = typer.Option(
    None,
    "--check-direct-dependency-maintenance-issues-only",
    help="Flag whether to only test only direct dependencies for maintenance issues",
)

_max_package_age_option = typer.Option(
    None, "--max-age-days", help="Maximum age threshold for packages in days"
)

_ignore_vulns_option = typer.Option(
    None,
    "--ignore-vulns",
    help="Comma-separated list of vulnerability IDs to ignore, e.g. VULN-123,VULN-456",
)

_config_option = typer.Option(
    None,
    "--config",
    help=(
        "Optional path to a configuration file (uv-secure.toml, .uv-secure.toml, or "
        "pyproject.toml)"
    ),
)

_version_option = typer.Option(
    None,
    "--version",
    callback=_version_callback,
    is_eager=True,
    help="Show the application's version",
)


_ignore_pkg_options = typer.Option(
    None,
    "--ignore-pkgs",
    metavar="PKG:SPEC1|SPEC2|…",
    help=(
        "Dependency with optional version specifiers. "
        "Syntax: name:spec1|spec2|…  "
        "e.g. foo:>=1.0,<1.5|==4.5.*"
    ),
)


@app.command()
def main(
    file_paths: list[Path] | None = _file_path_args,
    aliases: bool | None = _aliases_option,
    desc: bool | None = _desc_option,
    cache_path: Path = _cache_path_option,
    cache_ttl_seconds: float = _cache_ttl_seconds_option,
    disable_cache: bool = _disable_cache_option,
    forbid_yanked: bool | None = _forbid_yanked_option,
    max_package_age: int | None = _max_package_age_option,
    ignore_vulns: str | None = _ignore_vulns_option,
    ignore_pkgs: list[str] | None = _ignore_pkg_options,
    check_direct_dependency_vulnerabilities_only: bool
    | None = _check_direct_dependency_vulnerabilities_only_option,
    check_direct_dependency_maintenance_issues_only: bool
    | None = _check_direct_dependency_maintenance_issues_only_option,
    config_path: Path | None = _config_option,
    version: bool = _version_option,
) -> None:
    """Parse uv.lock files, check vulnerabilities, and display summary."""

    # Use uvloop or winloop if present
    try:
        if sys.platform in {"win32", "cygwin", "cli"}:
            from winloop import run
        else:
            from uvloop import run
    except ImportError:
        from asyncio import run

    run_status = run(
        check_lock_files(
            file_paths,
            aliases,
            desc,
            cache_path,
            cache_ttl_seconds,
            disable_cache,
            forbid_yanked,
            max_package_age,
            ignore_vulns,
            ignore_pkgs,
            check_direct_dependency_vulnerabilities_only,
            check_direct_dependency_maintenance_issues_only,
            config_path,
        )
    )
    if run_status == RunStatus.MAINTENANCE_ISSUES_FOUND:
        raise typer.Exit(code=1)
    if run_status == RunStatus.VULNERABILITIES_FOUND:
        raise typer.Exit(code=2)
    if run_status == RunStatus.RUNTIME_ERROR:
        raise typer.Exit(code=3)


if __name__ == "__main__":
    app()  # pragma: no cover
